module StarkitBanking
  module RBL
    module Account
      class GetMiniStatement
        include ApiClient

        private

        def service_name
          'GET TRANSACTIONS'
        end

        def invoke(env, req, callbacks)
          ApiBanking::AccountStatement.get_statement(env, req, callbacks)
        end

        def response(res)
          entries = Array.new

          res.transactions.each do |r|
            if (r.transactionType == "D" && r.amount.amount > 0) || (r.transactionType == "C" && r.amount.amount < 0)
              e = MiniStatement::Entry.new(r.transactionDateTime, r.amount, nil, r.narrative)
            else
              e = MiniStatement::Entry.new(r.transactionDateTime, nil, r.amount, r.narrative)
            end
            entries << e

          end

          entries
        end

        def credentials(payment_account)
          payment_account.credentials
        end

        def request(subscription, payment_account)          
          header = ApiBanking::AccountStatement::ReqHeader.new()
          reqBody = ApiBanking::AccountStatement::ReqBody.new()
          request = ApiBanking::AccountStatement::Request.new()
    
          header.corpID          = subscription.app_id
          header.approverID      = payment_account.created_by
    
          reqBody.accountNo = payment_account.account_number
          reqBody.transactionType = 'B'
          reqBody.fromDate = Date.today - 1.month
          reqBody.toDate = Date.today
    
          request.header = header
          request.body = reqBody
    
          request
        end

        def subscription(payment_account)
          payment_account.service_subscriptions.find_by(service_name: service_name)
        end
      end
    end
  end
end

