module StarkitBanking
  module RBL
    module Payment
      class ByFT2
        include ApiClient

        private

        def service_name
          'FUNDS TRANSFER'
        end

        def invoke(env, req, callbacks)
          ApiBanking::SinglePayment.transfer(env, req, callbacks)
        end

        def response(res)
          Transfer::Status.new(res.statusCode, res.bankReferenceNo, res.transferType)
        end

        def credentials(payment)
          payment.payment_account.credentials
        end

        def request(subscription, payment)
          
          remitter = ApiBanking::SinglePayment::Remitter.new()
          beneficiary = ApiBanking::SinglePayment::Beneficiary.new()
          header = ApiBanking::SinglePayment::ReqHeader.new()
          reqBody = ApiBanking::SinglePayment::ReqBody.new()
          request = ApiBanking::SinglePayment::Request.new()

          header.tranID          = payment.id.to_s
          header.corpID          = subscription.app_id
    
          reqBody.amount           = payment.transfer_amount.to_s
          reqBody.modeOfPay        = payment.transfer_type
          reqBody.remarks          = payment.transfer_description

          remitter.accountNo        = payment.payment_account.account_number
          remitter.accountName      = payment.payment_account.name
          remitter.accountIFSC      = payment.payment_account.ifsc_code
          remitter.mobileNo         = payment.payment_account.mobile_no
    
          beneficiary.accountIFSC      = payment.beneficiary_ifsc
          beneficiary.accountNo        = payment.beneficiary_account_number
          beneficiary.fullName         = payment.beneficiary_name
          beneficiary.address          = payment.beneficiary_address_line1
          beneficiary.email            = payment.beneficiary_email_id
          beneficiary.mobileNo         = payment.beneficiary_mobile_no

          reqBody.remitter = remitter
          reqBody.beneficiary = beneficiary
    
          request.header = header
          request.body = reqBody

          request
        end

        def subscription(payment)
          payment.payment_account.service_subscriptions.find_by(service_name: service_name)
        end

        def translate(element, value)
          case element
          when :bene_name
            value.slice(0,34)
          when :address_line1
            value.slice(0,49)
          when :transfer_description
            value.slice(0,49)
          end
        end

      end
    end
  end
end

