module StarkitBanking
  module YBL
    module Account
      class GetMiniStatement
        include ApiClient

        private

        def service_name
          'GET TRANSACTIONS'
        end

        def invoke(env, req, callbacks)
          ApiBanking::SocialBankingService.getTransactions(env, req, callbacks)
        end

        def response(res)
          entries = Array.new

          res.transactionsArray.each do |r|
            if (r.transactionType == "D" && r.amount > 0) || (r.transactionType == "C" && r.amount < 0)
              e = MiniStatement::Entry.new(r.recordDate, Money.new(r.amount.abs, r.currencyCode), nil, r.narrative)
            else
              e = MiniStatement::Entry.new(r.recordDate, nil, Money.new(r.amount.abs, r.currencyCode), r.narrative)
            end
            entries << e

          end

          entries
        end

        def credentials(payment_account)
          payment_account.credentials
        end

        def request(subscription, payment_account)
          customerIdentity = ApiBanking::SocialBankingService::GetTransactions::CustomerIdentity.new()
          accountIdentity = ApiBanking::SocialBankingService::GetTransactions::AccountIdentity.new()
          request = ApiBanking::SocialBankingService::GetTransactions::Request.new()

          customerIdentity.customerID = payment_account.customer_id

          accountIdentity.accountNo = payment_account.account_number

          request.appID = subscription.app_id

          request.customerIdentity = customerIdentity
          request.deviceID = rand(1000..9999).to_s
          request.accountIdentity = accountIdentity
          request.numTransactions = 10

          request
        end

        def subscription(payment_account)
          payment_account.service_subscriptions.find_by(service_name: service_name)
        end
      end
    end
  end
end

